from typing import cast

from _pytest.nodes import Item
from _pytest.reports import TestReport


def set_item_call_report(item: Item, report: TestReport) -> None:
    setattr(item, "call_report", report)


def get_item_call_report(item: Item) -> TestReport:
    return cast(TestReport, getattr(item, "call_report"))



# flake8: noqa
from .common import add_scenario_title_to_report
from .description_manager import DescriptionManager
from .severity import get_item_severity_level, set_severity_level
from .step_context_runner import StepContextNotDefinedError, StepContextRunner



from _pytest.nodes import Item

from overhave.pytest_plugin.helpers.bdd import get_scenario


def add_scenario_title_to_report(item: Item) -> None:
    item._obj.__allure_display_name__ = get_scenario(item).name  # type: ignore



# flake8: noqa
from .deps import get_description_manager, get_step_context_runner
from .helpers import DescriptionManager, StepContextNotDefinedError, StepContextRunner, get_scenario, has_issue_links
from .plugin_resolver import IPluginResolver, PluginResolver
from .proxy_manager import IProxyManager, ProxyManager, get_proxy_manager



from typing import Dict

from _pytest.nodes import Item


class BasePytestItemsCacheException(Exception):
    """Base exception for :class:`PytestItemsCache`."""


class DuplicateNodeIDError(BasePytestItemsCacheException):
    """Exception for situation with duplicated item nodeid."""


class NodeIDNotFoundError(BasePytestItemsCacheException):
    """Exception for situation with not existing item nodeid."""


class PytestItemsCache:
    """Class for Pytest items caching."""

    def __init__(self) -> None:
        self._items: Dict[str, Item] = {}

    def register_item(self, item: Item) -> None:
        if self._items.get(item.nodeid) is not None:
            raise DuplicateNodeIDError(f"Could not register item in cache: found duplicate nodeid='{item.nodeid}'!")
        self._items[item.nodeid] = item

    def get_item(self, nodeid: str) -> Item:
        item = self._items.get(nodeid)
        if item is None:
            raise NodeIDNotFoundError(f"Could not find item with nodeid='{nodeid}'!")
        return item
